#!/usr/bin/perl
#
# Program: lw360f.pl
# Version: 1.03
# Author: Michael Breuer (mbreuer@majjas.com)
# Copying: GPL
# 
# See the README file.
#
# *** CHANGES ***
# 1.03  Fixed typo in comment.
#       Added "kludge localhost" support.  If set, and uname support is enabled, then emails to the user are sent to <user>@localhost.
#       This corrected some local mail-alias issues and is not set by default.
# 1.02  Added support for scheduling and broadcast.
#	Added user name enhancements:
#	
#	***
#	User Name Support:
#		The sender name field is now assumed to match a valid local email ID.
#		Notifications are returned to the fax sender by using the name & substituting
#		and spaces with an underscore (_).  For example, if the user name "John Doe"
#		is entered as the sender name in the Adobe Postscript Fax support, then 
#		sendfax is told that the sender is "John Doe <John_Doe@{hostname of local host}>.
#		To make this work, you have to add appropriate mail aliases for your users.
#       Removed -v -v from sendfax line; misc fixes
# 1.01: Removed printf (debug code)
#       Fixed -c if no comment (faxcover)
#       Works with Hylafax 4.0b15 (will NOT work with 3.x).
#
#
# Todo...
#	Convert from sendfax to the new 'ftp' like protocol.
#	Add *all* cover page options (all supported by Apple and
#	by Faxcover.)
#
#
#**************************************************************
# These may be customized for your site.
$DEBUG=0;
$UNAME_SUPPORT=1;
$LOCALHOST_KLUDGE=0;
$TMP="/tmp";
$SENDFAX="/usr/local/bin/sendfax -v -v";
$FAXCOVER="/usr/local/bin/faxcover";
$CAT="/bin/cat";
$GS="/usr/bin/gs";
$RM="/bin/rm -f";
$LPR="/usr/bin/lpr -P ps ";
$HOSTNAME=`/bin/hostname`;
chop($HOSTNAME);
#
# You should not have to change anything below this line,
#
$TEMPCVR=	$TMP . "/__lw360_cvr$$";
$TEMPFAX=	$TMP . "/__lw360_fax$$";
$TMPCAPTURE=	$TMP . "/__lw360_tmp$$";
$start=1;
$lines=0;
$notify="-D";
$numr=0;
$thisr=-1;
$difforgs=0;
$i=0;
sub fixstring {
	local($s)=@_;
	$s=~s/\'/\'\\\'\'/g;
	return $s;	
}
sub lookupemailaddr {
	local($name)=@_;
	local($host);
#
# Ok... we're going to make a little convention (rather than build a new database).
# The rule is that the passed name MUST BE A VALID EMAIL ADDRESS when spaces are replaced
# by '_''s (ala cc:mail).  So, at the Mac, you can be "Joe Blow".  If so, you then require
# a corresponding smtp alias of 'Joe_Blow'.
#	
	$_=$name;
	$host=$HOSTNAME;
	if ($LOCALHOST_KLUDGE) {
		$host="localhost";
	}
	if ($UNAME_SUPPORT & / /) {
		s/ /_/g;
		return $name . " <$_\@" . $kludgehost . ">";
	}
	return $name . "@" . $kludgehost;
}

sub sendfax {
	local($options, $FILE)=@_;
	if ($DEBUG) {
		printf(STDERR "$SENDFAX $options $TEMPFAX 2>&1 |\n");
	}
	open(SENDFAX, "$SENDFAX $options $cmt $TEMPFAX |");
	while ($_=<SENDFAX>){printf STDERR "$_\n"}
	close(SENDFAX);
}
open(TEMPFILE, ">$TMPCAPTURE");
while ($start==1 && ($_=<STDIN>)) {
        printf(TEMPFILE $_);
	$lines++;
	chop($_);
	if (/^%[!^]PS-Adobe-3\.0/i) {
		$start=0;
	} else {
		@commands=split(/\//,$_);
		foreach (@commands) {
			s/^\\//;
			s/def//;
			if (/^DialCallee/) {
				$numr++;
				$thisr++;
			}
			if (/^SenderName/) {
				$thisr=0;
			}
			if (/^CoverNote/) {
				s/CoverNote\[//;
				s/\]//;
				s/\(//g;
				(@comments)=split(/\)/, $_);
			} else {
				s/\)//;
				if (/^MailingTime/) {s/\[/ /;s/\]//;}
				($tmp,$rest)=split(/\(/, $_,2);
				if (!$rest) {
					($tmp,$rest)=split(/ /, $tmp, 2);
				}
				if ($numr && ($tmp ne "RecipientName" || $rest ne "Group")) {
				   $cmd[$thisr]{$tmp}=$rest;
				}
				$_=$tmp;
			}
		}
	}
}

#
# Let's get the rest of the document
#

open(TEMPFAX,">$TEMPFAX");
$hdr="%!PS-Adobe-2.0 EPSF-2.0";
printf(TEMPFAX "$hdr\n");
while ($_=<STDIN>) {print(TEMPFAX $_);}
close(TEMPFAX);

#
# If it's not a fax, then just send to lpr as postscript
#
if ($lines==1) {
	`$LPR $TEMPFAX`;
	unlink $TEMPFAX;
	exit(0);
};
#
# OK, it's a fax... lets prepare parameters for sendfax
#

#
# format base sendfax options string
#

$options_base="-D -R ";
#
# If CoverSheet is null, then don't write a cover sheet.
#
if ($cmd[0]{'CoverSheet'}) {
	$_=&fixstring($cmd[0]{'CoverSheet'});
	if (/null/) { $options_base = $options_base . "-n ";}
}
#
# Get the sender's name (if present).
#
if ($cmd[0]{'SenderName'}) {
	$_=&fixstring($cmd[0]{'SenderName'});
#
# Make this a legal email address (Note: a corresponding alias is required for sendmail.
#
	$_=&lookupemailaddr($_);
	$options_base = $options_base . " " . "-f " . "\'" . $_ . "\'";
}
#
# Retries?
#
if ($cmd[0]{'MaxRetries'}) {
	$_=&fixstring($cmd[0]{'MaxRetries'});
   	s/'//g;
   	s/ //g;
   	if ($_ > 0) {$options_base = $options_base . " " . "-t " . $_;}
}
#
# Time to send (if specified, else immediate)
#
if ($cmd[0]{'MailingTime'}) {
	local(@monthtable)=( "January", "February", "March", "April", "May", "June", "July",
                             "August", "September", "October", "November", "December");
	local($yyyy, $mm, $dd, $hh, $min, $ss)=split(/ /, $cmd[0]{'MailingTime'});
	local($time);
	$options_base=sprintf("%s -a %02d%02d",
				$options_base,$hh,$min);
#	$options_base=sprintf("%s -a \'%02d:%02d %s %02d %04d\'",
#				$options_base,$hh,$min,$monthtable[--$mm],$dd,$yyyy);
}
#
# Build the comments string
#
foreach (@comments) {
        $cmt = $cmt . $_ . " "; # can't retain line breaks due to faxcover limitations
}
chop($cmt);
$cmt=&fixstring($cmt);
if (length($cmt)) {$cmt = "-c " . "'" . $cmt . "'";}
#
# Unused Stuff (future use?)
#
#	if ($cmd[0]{'SenderPhone'}) {$options_base = $options_base . " " . "-i " . $cmd[0]{'SenderPhone'};}
#	if ($cmd[0]{'RetryInterval'}) {$options_base_base = $options_base_base . " " . "-? " . $cmd[0]{'RetryInterval'};}
#	if ($cmd[0]{'FaxType'}) {$options_base = $options_base . " " . "-? " . $cmd[0]{'FaxType'};}
#
# *could* be used to set the number of pages for faxcover
#
#	if ($cmd[0]{'nPages'}) {
#		$_ = $cmd[0]{'nPages'};
#		s/ //g;
#		s/'//g;
#		$options_base = $options_base . " " . "-p " . $cmd[0]{'nPages'};}

#
# now lets format the recipient info...
# It's in a loop as there might be more than one recipient (group fax).
#

#
# First, let's see how much work *really* has to be done...
#
$rorg=&fixstring($cmd[0]{'RecipientOrg'});
for($i=1;$i<$numr;$i++) {
	if ($rorg ne $cmd[0]{'RecipientOrg'}) {
		$difforgs=1;
		break;
	}
}
if (!$difforgs && $rorg) {
	$options_base = $options_base . " " . "-x " . "\'" . $rorg . "\'";
}
$sfoptions= $cmt . " " . $options_base;
for($i=0;$i<$numr;$i++) {
#
# if we're sending different fax covers for different recipients, then we need to reset the base.
#
	if ($difforgs) {$sfoptions=$options_base;}
	if ($difforgs && $cmd[$i]{'RecipientOrg'}) {
		$sfoptions = $sfoptions . " " . "-x \'" . &fixstring($cmd[$i]{'RecipientOrg'}) . "\'";
	}
	if ($cmd[$i]{'DialCallee'}) {
   		if ($cmd[$i]{'RecipientName'}) {
   			$sfoptions = $sfoptions . " -d \'";
			$sfoptions = $sfoptions . &fixstring($cmd[$i]{'RecipientName'}) . "\@";
			$_=&fixstring($cmd[$i]{'DialCallee'});
   			s/T//;
   			s/'//g;
   			$sfoptions= $sfoptions . $_ . "\'";
		}
	}
	if ($difforgs) {&sendfax($sfoptions, $TEMPFAX);}
}
if (!$difforgs) {&sendfax($sfoptions, $cmt, $TEMPFAX);}
unlink $TEMPCVR;
unlink $TEMPFAX;
